@@ -1,5 +1,5 @@
 ;;; Simple IMAP4 (RFC 2060) client for VM
-;;; Copyright (C) 1998 Kyle E. Jones
+;;; Copyright (C) 1998, 2001 Kyle E. Jones
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 	(imap-retrieved-messages vm-imap-retrieved-messages)
 	(did-delete nil)
 	(source-nopwd (vm-imapdrop-sans-password source))
+	(use-rfc822-peek nil)
 	auto-expunge x select source-list uid
 	can-delete read-write uid-validity
 	mailbox mailbox-count message-size response
@@ -133,22 +134,25 @@
 		      (throw 'skip t)))
 		(message "Retrieving message %d (of %d) from %s..."
 			 n mailbox-count imapdrop)
-		(condition-case data
+		(if use-rfc822-peek
 		    (progn
-		      (vm-imap-send-command process
-					    (format "FETCH %d (BODY.PEEK[])"
-						    n))
-		      (vm-imap-retrieve-to-crashbox process destination
-						    statblob t))
-		  (vm-imap-protocol-error
-		   (message "FETCH %d (BODY.PEEK[]) failed, trying RFC822.PEEK"
-			    n)
-		   (vm-imap-send-command process
-					 (format
-					  "FETCH %d (RFC822.PEEK)" n))
-		   (vm-imap-retrieve-to-crashbox process destination
-						 statblob nil)))
-
+		       (vm-imap-send-command process
+					     (format
+					      "FETCH %d (RFC822.PEEK)" n))
+		       (vm-imap-retrieve-to-crashbox process destination
+						     statblob nil))
+		  (condition-case data
+		      (progn
+			(vm-imap-send-command process
+					      (format "FETCH %d (BODY.PEEK[])"
+						      n))
+			(vm-imap-retrieve-to-crashbox process destination
+						      statblob t))
+		    (vm-imap-protocol-error
+		     (vm-imap-send-command process
+					   (format "FETCH %d (RFC822.PEEK)" n))
+		     (vm-imap-retrieve-to-crashbox process destination
+						   statblob nil))))
 		(vm-increment retrieved)
 		(and b-per-session
 		     (setq retrieved-bytes (+ retrieved-bytes message-size)))
@@ -389,6 +393,10 @@ on all the relevant IMAP servers and then immediately expunges."
 	(imapdrop (vm-safe-imapdrop-string source))
 	(coding-system-for-read (vm-binary-coding-system))
 	(coding-system-for-write (vm-binary-coding-system))
+	(use-ssl nil)
+	(use-ssh nil)
+	(session-name "IMAP")
+	(process-connection-type nil)
 	greeting timestamp
 	host port mailbox auth user pass source-list process-buffer
 	source-nopwd-nombox)
@@ -404,6 +412,16 @@ on all the relevant IMAP servers and then immediately expunges."
 		pass (nth 6 source-list)
 		source-nopwd-nombox
 		(vm-imapdrop-sans-password-and-mailbox source))
+	  (cond ((equal "imap-ssl" (car source-list))
+		 (setq use-ssl t
+		       session-name "IMAP over SSL")
+		 (if (null vm-stunnel-program)
+		     (error "vm-stunnel-program must be non-nil to use IMAP over SSL.")))
+		((equal "imap-ssh" (car source-list))
+		 (setq use-ssh t
+		       session-name "IMAP over SSH")
+		 (if (null vm-ssh-program)
+		     (error "vm-ssh-program must be non-nil to use IMAP over SSH."))))
 	  ;; carp if parts are missing
 	  (if (null host)
 	      (error "No host in IMAP maildrop specification, \"%s\""
@@ -441,11 +459,13 @@ on all the relevant IMAP servers and then immediately expunges."
 					    vm-imap-passwords)))
 	  ;; get the trace buffer
 	  (setq process-buffer
-		(vm-make-work-buffer (format "trace of IMAP session to %s"
+		(vm-make-work-buffer (format "trace of %s session to %s"
+					     session-name
 					     host)))
 	  (save-excursion
 	    (set-buffer process-buffer)
 	    (buffer-disable-undo process-buffer)
+	    (make-local-variable 'vm-imap-read-point)
 	    ;; clear the trace buffer of old output
 	    (erase-buffer)
 	    ;; Tell MULE not to mess with the text.
@@ -458,16 +478,32 @@ on all the relevant IMAP servers and then immediately expunges."
 							user pass)))
 	    (if (processp process)
 		(set-process-buffer process (current-buffer))
-	      (insert "starting IMAP session " (current-time-string) "\n")
+	      (insert "starting " session-name
+		      " session " (current-time-string) "\n")
 	      (insert (format "connecting to %s:%s\n" host port))
 	      ;; open the connection to the server
-	      (setq process (open-network-stream "IMAP" process-buffer
-						 host port))
+	      (cond (use-ssl
+		     (vm-setup-stunnel-random-data-if-needed)
+		     (setq process
+			   (apply 'start-process session-name process-buffer
+				  vm-stunnel-program
+				  (nconc (vm-stunnel-random-data-args)
+					 (list "-W" "-c" "-r"
+					       (format "%s:%s" host port))
+					 vm-stunnel-program-switches))))
+		    (use-ssh
+		     (setq process (open-network-stream
+				    session-name process-buffer
+				    "127.0.0.1"
+				    (vm-setup-ssh-tunnel host port))))
+		    (t
+		     (setq process (open-network-stream session-name
+							process-buffer
+							host port))))
 	      (and (null process) (throw 'end-of-session nil))
-	      (insert "connected\n"))
-	    (process-kill-without-query process)
-	    (make-local-variable 'vm-imap-read-point)
+	      (insert-before-markers "connected\n"))
 	    (setq vm-imap-read-point (point))
+	    (process-kill-without-query process)
 	    (if (null (setq greeting (vm-imap-read-greeting process)))
 		(progn (delete-process process)
 		       (throw 'end-of-session nil)))
@@ -521,7 +557,7 @@ on all the relevant IMAP servers and then immediately expunges."
 			       (concat
 				(vm-xor-string secret ipad) challenge)))))
 			   answer (vm-mime-base64-encode-string answer))
-		     (vm-imap-send-command process answer)
+		     (vm-imap-send-command process answer nil t)
 		     (and (null (vm-imap-read-ok-response process))
 			  (progn
 			    (setq vm-imap-passwords
@@ -544,7 +580,8 @@ on all the relevant IMAP servers and then immediately expunges."
 	    (setq process-to-shutdown nil)
 	    process ))
       (if process-to-shutdown
-	  (vm-imap-end-session process-to-shutdown t)))))
+	  (vm-imap-end-session process-to-shutdown t))
+      (vm-tear-down-stunnel-random-data))))
 
 (defun vm-imap-end-session (process &optional keep-buffer)
   (save-excursion
@@ -564,42 +601,47 @@ on all the relevant IMAP servers and then immediately expunges."
       (run-at-time 2 nil 'delete-process process))))
 
 (defun vm-imap-stat-timer (o) (aref o 0))
-(defun vm-imap-stat-x-box (o) (aref o 1))
-(defun vm-imap-stat-x-currmsg (o) (aref o 2))
-(defun vm-imap-stat-x-maxmsg (o) (aref o 3))
-(defun vm-imap-stat-x-got (o) (aref o 4))
-(defun vm-imap-stat-x-need (o) (aref o 5))
-(defun vm-imap-stat-y-box (o) (aref o 6))
-(defun vm-imap-stat-y-currmsg (o) (aref o 7))
-(defun vm-imap-stat-y-maxmsg (o) (aref o 8))
-(defun vm-imap-stat-y-got (o) (aref o 9))
-(defun vm-imap-stat-y-need (o) (aref o 10))
+(defun vm-imap-stat-did-report (o) (aref o 1))
+(defun vm-imap-stat-x-box (o) (aref o 2))
+(defun vm-imap-stat-x-currmsg (o) (aref o 3))
+(defun vm-imap-stat-x-maxmsg (o) (aref o 4))
+(defun vm-imap-stat-x-got (o) (aref o 5))
+(defun vm-imap-stat-x-need (o) (aref o 6))
+(defun vm-imap-stat-y-box (o) (aref o 7))
+(defun vm-imap-stat-y-currmsg (o) (aref o 8))
+(defun vm-imap-stat-y-maxmsg (o) (aref o 9))
+(defun vm-imap-stat-y-got (o) (aref o 10))
+(defun vm-imap-stat-y-need (o) (aref o 11))
 
 (defun vm-set-imap-stat-timer (o val) (aset o 0 val))
-(defun vm-set-imap-stat-x-box (o val) (aset o 1 val))
-(defun vm-set-imap-stat-x-currmsg (o val) (aset o 2 val))
-(defun vm-set-imap-stat-x-maxmsg (o val) (aset o 3 val))
-(defun vm-set-imap-stat-x-got (o val) (aset o 4 val))
-(defun vm-set-imap-stat-x-need (o val) (aset o 5 val))
-(defun vm-set-imap-stat-y-box (o val) (aset o 6 val))
-(defun vm-set-imap-stat-y-currmsg (o val) (aset o 7 val))
-(defun vm-set-imap-stat-y-maxmsg (o val) (aset o 8 val))
-(defun vm-set-imap-stat-y-got (o val) (aset o 9 val))
-(defun vm-set-imap-stat-y-need (o val) (aset o 10 val))
+(defun vm-set-imap-stat-did-report (o val) (aset o 1 val))
+(defun vm-set-imap-stat-x-box (o val) (aset o 2 val))
+(defun vm-set-imap-stat-x-currmsg (o val) (aset o 3 val))
+(defun vm-set-imap-stat-x-maxmsg (o val) (aset o 4 val))
+(defun vm-set-imap-stat-x-got (o val) (aset o 5 val))
+(defun vm-set-imap-stat-x-need (o val) (aset o 6 val))
+(defun vm-set-imap-stat-y-box (o val) (aset o 7 val))
+(defun vm-set-imap-stat-y-currmsg (o val) (aset o 8 val))
+(defun vm-set-imap-stat-y-maxmsg (o val) (aset o 9 val))
+(defun vm-set-imap-stat-y-got (o val) (aset o 10 val))
+(defun vm-set-imap-stat-y-need (o val) (aset o 11 val))
 
 (defun vm-imap-start-status-timer ()
-  (let ((blob (make-vector 11 nil))
+  (let ((blob (make-vector 12 nil))
 	timer)
     (setq timer (add-timeout 5 'vm-imap-report-retrieval-status blob 5))
     (vm-set-imap-stat-timer blob timer)
     blob ))
 
 (defun vm-imap-stop-status-timer (status-blob)
+  (if (vm-imap-stat-did-report status-blob)
+      (message ""))
   (if (fboundp 'disable-timeout)
       (disable-timeout (vm-imap-stat-timer status-blob))
     (cancel-timer (vm-imap-stat-timer status-blob))))
 
 (defun vm-imap-report-retrieval-status (o)
+  (vm-set-imap-stat-did-report o t)
   (cond ((null (vm-imap-stat-x-got o)) t)
 	;; should not be possible, but better safe...
 	((not (eq (vm-imap-stat-x-box o) (vm-imap-stat-y-box o))) t)
@@ -608,34 +650,38 @@ on all the relevant IMAP servers and then immediately expunges."
 		    (vm-imap-stat-x-currmsg o)
 		    (vm-imap-stat-x-maxmsg o)
 		    (vm-imap-stat-x-box o)
-		    (format "%d%s of %d%s"
-			    (vm-imap-stat-x-got o)
-			    (if (> (vm-imap-stat-x-got o)
-				   (vm-imap-stat-x-need o))
-				"!"
-			      "")
-			    (vm-imap-stat-x-need o)
-			    (if (eq (vm-imap-stat-x-got o)
-				    (vm-imap-stat-y-got o))
-				" (stalled)"
-			      "")))))
+		    (if (vm-imap-stat-x-need o)
+			(format "%d%s of %d%s"
+				(vm-imap-stat-x-got o)
+				(if (> (vm-imap-stat-x-got o)
+				       (vm-imap-stat-x-need o))
+				    "!"
+				  "")
+				(vm-imap-stat-x-need o)
+				(if (eq (vm-imap-stat-x-got o)
+					(vm-imap-stat-y-got o))
+				    " (stalled)"
+				  ""))
+		      "post processing"))))
   (vm-set-imap-stat-y-box o (vm-imap-stat-x-box o))
   (vm-set-imap-stat-y-currmsg o (vm-imap-stat-x-currmsg o))
   (vm-set-imap-stat-y-maxmsg o (vm-imap-stat-x-maxmsg o))
   (vm-set-imap-stat-y-got o (vm-imap-stat-x-got o))
   (vm-set-imap-stat-y-need o (vm-imap-stat-x-need o)))
 
-(defun vm-imap-send-command (process command)
+(defun vm-imap-send-command (process command &optional tag no-tag)
   (goto-char (point-max))
-  (insert-before-markers "VM ")
+  (or no-tag (insert-before-markers (or tag "VM") " "))
   (let ((case-fold-search t))
     (if (string-match "^LOGIN" command)
 	(insert-before-markers "LOGIN <parameters omitted>\r\n")
       (insert-before-markers command "\r\n")))
   (setq vm-imap-read-point (point))
-  (process-send-string process "VM ")
-  (process-send-string process command)
-  (process-send-string process "\r\n"))
+  ;; previously we had a process-send-string call for each string
+  ;; to avoid extra consing but that caused a lot of packet overhead.
+  (if no-tag
+      (process-send-string process (format "%s\r\n" command))
+    (process-send-string process (format "%s %s\r\n" (or tag "VM") command))))
 
 (defun vm-imap-select-mailbox (process mailbox &optional just-examine)
   (let ((imap-buffer (current-buffer))
@@ -737,7 +783,7 @@ on all the relevant IMAP servers and then immediately expunges."
 	      (cond ((vm-imap-response-matches response '* 'atom 'FETCH 'list)
 		     (setq fetch-response response
 			   need-header nil))
-		    ((vm-imap-response-matches response 'VM 'OK 'FETCH)
+		    ((vm-imap-response-matches response 'VM 'OK)
 		     (setq need-ok nil))))
 	    (if need-header
 		(vm-imap-protocol-error "FETCH OK sent before FETCH response"))
@@ -803,7 +849,6 @@ on all the relevant IMAP servers and then immediately expunges."
     ;; must make the read point a marker so that it stays fixed
     ;; relative to the text when we modify things below.
     (setq vm-imap-read-point (point-marker))
-    (vm-set-imap-stat-x-got statblob nil)
     (setq list (cdr (nth 3 fetch-response)))
     (cond
      (bodypeek
@@ -820,9 +865,11 @@ on all the relevant IMAP servers and then immediately expunges."
 	    start (nth 1 p))))
     (goto-char (nth 2 p))
     (setq end (point-marker))
+    (vm-set-imap-stat-x-need statblob nil)
     (vm-imap-cleanup-region start end)
     (vm-munge-message-separators vm-folder-type start end)
     (goto-char start)
+    (vm-set-imap-stat-x-got statblob nil)
     ;; avoid the consing and stat() call for all but babyl
     ;; files, since this will probably slow things down.
     ;; only babyl files have the folder header, and we
@@ -918,16 +965,12 @@ on all the relevant IMAP servers and then immediately expunges."
     retval ))
 
 (defun vm-imap-cleanup-region (start end)
-  (if (> (- end start) 30000)
-      (message "CRLF conversion and char unstuffing..."))
   (setq end (vm-marker end))
   (save-excursion
     (goto-char start)
     ;; CRLF -> LF
     (while (and (< (point) end) (search-forward "\r\n"  end t))
       (replace-match "\n" t t)))
-  (if (> (- end start) 30000)
-      (message "CRLF conversion and char unstuffing... done"))
   (set-marker end nil))
 
 (defun vm-imapdrop-sans-password (source)
@@ -1037,10 +1080,12 @@ on all the relevant IMAP servers and then immediately expunges."
 		 (skip-chars-forward "^\042")
 		 (setq curpoint (point))
 		 (if (looking-at "\042")
-		     (setq done t)
+		     (progn
+		       (setq done t)
+		       (forward-char 1))
 		   (accept-process-output process)
 		   (goto-char curpoint))
-	       (setq token (list 'string start (1- curpoint))))))
+	       (setq token (list 'string start curpoint)))))
 	    ;; should be (looking-at "[\000-\040\177-\377]")
 	    ;; but Microsoft Exchange emits 8-bit chars.
 	    ((looking-at "[\000-\040\177]")
