@@ -1,5 +1,5 @@
 ;;; Mailing, forwarding, and replying commands for VM
-;;; Copyright (C) 1989-1998 Kyle E. Jones
+;;; Copyright (C) 1989-2001 Kyle E. Jones
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -480,6 +480,18 @@ as having been replied to, if appropriate."
 	      (setq resent t))
 	  (vm-mail-mode-remove-header "Date:")))))
 
+(defvar vm-dont-ask-coding-system-question nil)
+
+(cond ((and vm-fsfemacs-mule-p
+	    (fboundp 'select-message-coding-system)
+	    (not (fboundp 'vm-old-select-message-coding-system)))
+       (fset 'vm-old-select-message-coding-system
+	     (symbol-function 'select-message-coding-system))
+       (defun select-message-coding-system (&rest ignored)
+	 (if vm-dont-ask-coding-system-question
+	     nil
+	   (apply 'vm-old-select-message-coding-system ignored)))))
+
 (defun vm-mail-send ()
   "Just like mail-send except that VM flags the appropriate message(s)
 as replied to, forwarded, etc, if appropriate."
@@ -528,7 +540,11 @@ as replied to, forwarded, etc, if appropriate."
       ;; save-excursion to be sure.
       ;;
       ;; also protect value of this-command from minibuffer reads
-      (let ((this-command this-command))
+      (let ((this-command this-command)
+	    ;; For Emacs 21.
+	    (mail-send-nonascii t)
+	    (sendmail-coding-system (vm-binary-coding-system))
+	    (vm-dont-ask-coding-system-question t))
 	(save-excursion
 	  (mail-send))))
     ;; be careful, something could have killed the composition
@@ -1091,12 +1107,47 @@ found, the current buffer remains selected."
 		      '(vm-continue-composing-message composing-message)))
       (message "No composition buffers found"))))
 
+;;;###autoload
 (defun vm-mail-to-mailto-url (url)
-  (let ((address (car (vm-parse url "^mailto:\\(.+\\)"))))
-    (setq address (vm-url-decode-string address))
-    (vm-select-folder-buffer)
-    (vm-check-for-killed-summary)
-    (vm-mail-internal nil address)
+  (vm-session-initialization)
+  (vm-check-for-killed-folder)
+  (vm-select-folder-buffer-if-possible)
+  (vm-check-for-killed-summary)
+  (let ((list (vm-parse url "^mailto:\\([^?]+\\)\\??\\|\\([^&]+\\)&?"
+			'(1 2)))
+	to subject in-reply-to cc references newsgroups body
+	tem header value header-list)
+    (setq to (car list)
+	  to (vm-url-decode-string to)
+	  list (cdr list))
+    (while list
+      (setq tem (vm-parse (car list) "\\([^=]+\\)=?"))
+      (if (null (nth 1 tem))
+	  nil
+	(setq header (downcase (vm-url-decode-string (car tem)))
+	      value (vm-url-decode-string (nth 1 tem)))
+	(if (member header '("subject" "in-reply-to" "cc"
+			     "references" "newsgroups" "body"))
+	    ;; set the variable let-bound above
+	    (set (intern header) value)
+	  ;; we'll insert the header later
+	  (setq header-list (cons header (cons value header-list)))))
+      (setq list (cdr list)))
+    (vm-mail-internal nil to subject in-reply-to cc references newsgroups)
+    (save-excursion
+      (goto-char (point-min))
+      (while header-list
+	(insert (car header-list) ": ")
+	(capitalize-region (point) (save-excursion (beginning-of-line) (point)))
+	(insert (nth 1 header-list) "\n")
+	(setq header-list (nthcdr 2 header-list)))
+      (if (null body)
+	  nil
+	(mail-text)
+	(save-excursion (insert (vm-url-decode-string body) "\n"))
+	;; CRLF to LF for line breaks in the body
+	(while (search-forward "\r\n" nil t)
+	  (replace-match "\n"))))
     (run-hooks 'vm-mail-hook)
     (run-hooks 'vm-mail-mode-hook)))
 
@@ -1179,16 +1230,26 @@ found, the current buffer remains selected."
     (if (not (= (preceding-char) ?\n))
 	(insert ?\n))
     (insert mail-header-separator "\n")
-    (cond ((stringp mail-signature)
-	   (save-excursion
-	     (insert mail-signature)))
-	  ((eq mail-signature t)
-	   (save-excursion
-	     (insert "\n-- \n")
-	     (insert-file-contents (or (and (boundp 'mail-signature-file)
-					    (stringp mail-signature-file)
-					    mail-signature-file)
-				       "~/.signature")))))
+    (if mail-signature
+	(save-excursion
+	  (save-restriction
+	    (narrow-to-region (point) (point))
+	    (cond ((stringp mail-signature)
+		   (insert mail-signature))
+		  ((eq mail-signature t)
+		   (insert-file-contents (or (and (boundp 'mail-signature-file)
+						  (stringp mail-signature-file)
+						  mail-signature-file)
+					     "~/.signature")))
+		  (t
+		   (let ((str (eval mail-signature))) 
+		     (if (stringp str)
+			 (insert str)))))
+	    (goto-char (point-min))
+	    (if (looking-at "\n*-- \n")
+		nil
+	      (insert "\n-- \n"))
+	    (goto-char (point-max)))))
     ;; move this buffer to the head of the buffer list so window
     ;; config stuff will select it as the composition buffer.
     (vm-unbury-buffer (current-buffer))
@@ -1230,6 +1291,10 @@ found, the current buffer remains selected."
 		(list this-command 'composing-message))
     (if (null to)
 	(mail-position-on-field "To"))
+    (if (boundp 'post-command-idle-hook)
+	(add-hook 'post-command-idle-hook
+		   'vm-update-composition-buffer-name t)
+      (add-hook 'post-command-hook 'vm-update-composition-buffer-name t))
     (run-hooks 'mail-setup-hook)))
 
 (defun vm-reply-other-frame (count)
@@ -1402,10 +1467,11 @@ message."
 	       (null (vm-mail-mode-get-header-contents "MIME-Version:"))
 	       (vm-mime-encode-composition))
 	  (vm-remove-mail-mode-header-separator)
+	  (vm-munge-message-separators 'mmdf (point-min) (point-max))
 	  (goto-char (point-min))
-	  (insert (vm-leading-message-separator vm-default-From_-folder-type))
+	  (insert (vm-leading-message-separator 'mmdf))
 	  (goto-char (point-max))
-	  (insert (vm-trailing-message-separator vm-default-From_-folder-type))
+	  (insert (vm-trailing-message-separator 'mmdf))
 	  (set-buffer-modified-p nil)
 	  ;; point of no return, don't kill it if the user quits
 	  (setq temp-buffer nil)
@@ -1423,6 +1489,31 @@ message."
 		      (list this-command) '(vm-mode startup)))
       (and temp-buffer (kill-buffer temp-buffer)))))
 
+(defun vm-update-composition-buffer-name ()
+  (if (and (eq major-mode 'mail-mode)
+           (save-match-data (string-match "^\\(mail\\|reply\\) to "
+					  (buffer-name))))
+      (let ((to (mail-fetch-field "To"))
+            (cc (mail-fetch-field "Cc"))
+	    (curbufname (buffer-name))
+	    (deactivate-mark)
+	    fmt newbufname
+            (ellipsis ""))
+	(cond (vm-reply-list (setq fmt "reply to %s%s"))
+	      (t (setq fmt "mail to %s%s")))
+        (setq to (vm-parse-addresses to)
+              cc (vm-parse-addresses cc))
+        (if (or (cdr to)
+                (and (car to) (car cc)))
+            (setq ellipsis ", ..."))
+        (setq newbufname (or (car to) (car cc) "foo (?)")
+              newbufname (funcall vm-chop-full-name-function newbufname)
+              newbufname (or (car newbufname) (car (cdr newbufname)))
+              newbufname (format fmt newbufname ellipsis))
+        (if (equal newbufname curbufname)
+            nil
+          (rename-buffer newbufname t)))))
+
 (defun vm-mail-mode-remove-tm-hooks ()
   (remove-hook 'mail-setup-hook 'turn-on-mime-edit)
   (remove-hook 'mail-setup-hook 'mime/decode-message-header)
