@@ -20,7 +20,7 @@
 (defvar enable-multibyte-characters)
 
 ;;;###autoload
-(defun vm (&optional folder read-only)
+(defun vm (&optional folder read-only access-method)
   "Read mail under Emacs.
 Optional first arg FOLDER specifies the folder to visit.  It defaults
 to the value of vm-primary-inbox.  The folder buffer is put into VM
@@ -50,7 +50,14 @@ See the documentation for vm-mode for more information."
     (let ((full-startup (not (bufferp folder)))
 	  (did-read-index-file nil)
 	  folder-buffer first-time totals-blurb
+	  folder-name remote-spec
 	  preserve-auto-save-file)
+      (cond ((eq access-method 'pop)
+	     (setq remote-spec (vm-pop-find-spec-for-name folder))
+	     (if (null remote-spec)
+		 (error "No such POP folder: %s" folder))
+	     (setq folder-name folder
+		   folder (vm-pop-make-filename-for-spec remote-spec))))
       (setq folder-buffer
 	    (if (bufferp folder)
 		folder
@@ -75,12 +82,16 @@ See the documentation for vm-mode for more information."
 			(message "Reading %s..." file)
 			(prog1 (find-file-noselect file)
 			  ;; update folder history
-			  (let ((item (or folder vm-primary-inbox)))
+			  (let ((item (or remote-spec folder
+					  vm-primary-inbox)))
 			    (if (not (equal item (car vm-folder-history)))
 				(setq vm-folder-history
 				      (cons item vm-folder-history))))
 			  (message "Reading %s... done" file))))))))
       (set-buffer folder-buffer)
+      (cond ((eq access-method 'pop)
+	     (if (not (equal folder-name (buffer-name)))
+		 (rename-buffer folder-name t))))
       (if (and vm-fsfemacs-mule-p enable-multibyte-characters)
 	  (set-buffer-multibyte nil))
       ;; for MULE
@@ -171,7 +182,9 @@ See the documentation for vm-mode for more information."
 	    ;; user's default face charset, rather than as octal
 	    ;; escapes.
 	    (vm-fsfemacs-nonmule-display-8bit-chars)
-	    (vm-mode-internal)
+	    (vm-mode-internal access-method)
+	    (cond ((eq access-method 'pop)
+		   (vm-set-folder-pop-maildrop-spec remote-spec)))
 	    ;; If the buffer is modified we don't know if the
 	    ;; folder format has been changed to be different
 	    ;; from index file, so don't read the index file in
@@ -228,12 +241,7 @@ See the documentation for vm-mode for more information."
       (vm-update-summary-and-mode-line)
       ;; need to do this after any frame creation because the
       ;; toolbar sets frame-specific height and width specifiers.
-      (and (vm-toolbar-support-possible-p) vm-use-toolbar
-	   (progn
-	     (message "Initializing toolbar...")
-	     (vm-toolbar-install-toolbar)
-	     (message "Initializing toolbar... done")
-	     (vm-toolbar-update-toolbar)))
+      (vm-toolbar-install-or-uninstall-toolbar)
 
       (and vm-use-menus (vm-menu-support-possible-p)
 	   (vm-menu-install-visited-folders-menu))
@@ -275,8 +283,6 @@ See the documentation for vm-mode for more information."
 
       (run-hooks 'vm-visit-folder-hook)
 
-      (if full-startup
-	  (message totals-blurb))
       ;; Warn user about auto save file, if appropriate.
       (if (and full-startup preserve-auto-save-file)
 	  (message 
@@ -287,13 +293,17 @@ See the documentation for vm-mode for more information."
       ;; stop here.
       (if (or (not full-startup) preserve-auto-save-file)
 	  (throw 'done t))
+      
+      (if full-startup
+	  (message totals-blurb))
+
       (if (and vm-auto-get-new-mail
 	       (not vm-block-new-mail)
 	       (not vm-folder-read-only))
 	  (progn
 	    (message "Checking for new mail for %s..."
 		     (or buffer-file-name (buffer-name)))
-	    (if (and (vm-get-spooled-mail t) (vm-assimilate-new-messages t))
+	    (if (vm-get-spooled-mail t)
 		(progn
 		  (setq totals-blurb (vm-emit-totals-blurb))
 		  (if (vm-thoughtfully-select-message)
@@ -342,7 +352,7 @@ See the documentation for vm-mode for more information."
 (defun vm-mode (&optional read-only)
   "Major mode for reading mail.
 
-This is VM 6.95.
+This is VM 7.04.
 
 Commands:
    h - summarize folder contents
@@ -546,8 +556,12 @@ Variables:
    vm-frame-per-summary
    vm-highlighted-header-face
    vm-highlighted-header-regexp
+   vm-honor-mime-content-disposition
    vm-honor-page-delimiters
+   vm-icontopbm-program
    vm-image-directory
+   vm-imagemagick-convert-program
+   vm-imagemagick-identify-program
    vm-imap-auto-expunge-alist
    vm-imap-bytes-per-session
    vm-imap-expunge-after-retrieving
@@ -588,6 +602,7 @@ Variables:
    vm-mime-base64-encoder-switches
    vm-mime-button-face
    vm-mime-button-format-alist
+   vm-mime-charset-converter-alist
    vm-mime-charset-font-alist
    vm-mime-confirm-delete
    vm-mime-decode-for-preview
@@ -607,7 +622,9 @@ Variables:
    vm-mime-qp-decoder-switches
    vm-mime-qp-encoder-program
    vm-mime-qp-encoder-switches
+   vm-mime-require-mime-version-header
    vm-mime-type-converter-alist
+   vm-mime-use-image-strips
    vm-mime-uuencode-decoder-program
    vm-mime-uuencode-decoder-switches
    vm-mode-hook
@@ -626,6 +643,7 @@ Variables:
    vm-pop-auto-expunge-alist
    vm-pop-bytes-per-session
    vm-pop-expunge-after-retrieving
+   vm-pop-folder-alist
    vm-pop-max-message-size
    vm-pop-md5-program
    vm-pop-messages-per-session
@@ -663,8 +681,14 @@ Variables:
    vm-spool-file-suffixes
    vm-spool-files
    vm-spooled-mail-waiting-hook
+   vm-ssh-program
+   vm-ssh-program-switches
+   vm-ssh-remote-command
    vm-startup-with-summary
    vm-strip-reply-headers
+   vm-stunnel-program
+   vm-stunnel-program-switches
+   vm-stunnel-random-data-method
    vm-subject-significant-chars
    vm-summary-arrow
    vm-summary-format
@@ -678,6 +702,7 @@ Variables:
    vm-thread-using-subject
    vm-toolbar-pixmap-directory
    vm-trust-From_-with-Content-Length
+   vm-uncompface-program
    vm-undisplay-buffer-hook
    vm-unforwarded-header-regexp
    vm-url-browser
@@ -735,9 +760,16 @@ visited folder."
   (vm-select-folder-buffer-if-possible)
   (vm-check-for-killed-summary)
   (setq vm-last-visit-folder folder)
-  (let ((default-directory (or vm-folder-directory default-directory)))
-    (setq folder (expand-file-name folder)))
-  (vm folder read-only))
+  (let ((access-method nil) foo)
+    (cond ((and (stringp vm-recognize-pop-maildrops)
+		(string-match vm-recognize-pop-maildrops folder)
+		(setq foo (vm-pop-find-name-for-spec folder)))
+	   (setq folder foo
+		 access-method 'pop))
+	  (t
+	   (let ((default-directory (or vm-folder-directory default-directory)))
+	     (setq folder (expand-file-name folder)))))
+    (vm folder read-only access-method)))
 
 ;;;###autoload
 (defun vm-visit-folder-other-frame (folder &optional read-only)
@@ -800,6 +832,108 @@ visited folder."
 	(vm-search-other-frames nil))
     (vm-visit-folder folder read-only)))
 
+;;;###autoload
+(defun vm-visit-pop-folder (folder &optional read-only)
+  "Visit a POP mailbox.
+VM will parse and present its messages to you in the usual way.
+
+First arg FOLDER specifies the name of the POP mailbox to visit.
+You can only visit mailboxes that are specified in `vm-pop-folder-alist'.
+When this command is called interactively the mailbox name is read from the
+minibuffer.
+
+Prefix arg or optional second arg READ-ONLY non-nil indicates
+that the folder should be considered read only.  No attribute
+changes, messages additions or deletions will be allowed in the
+visited folder."
+  (interactive
+   (save-excursion
+     (vm-session-initialization)
+     (vm-check-for-killed-folder)
+     (vm-select-folder-buffer-if-possible)
+     (let ((completion-list (mapcar (function (lambda (x) (nth 1 x)))
+				    vm-pop-folder-alist))
+	   (default vm-last-visit-pop-folder)
+	   (this-command this-command)
+	   (last-command last-command))
+       (list (vm-read-string
+	      (format "Visit%s POP folder:%s "
+		      (if current-prefix-arg " read only" "")
+		      (if default
+			  (format " (default %s)" default)
+			""))
+	      completion-list)
+	     current-prefix-arg))))
+  (vm-session-initialization)
+  (vm-check-for-killed-folder)
+  (vm-select-folder-buffer-if-possible)
+  (vm-check-for-killed-summary)
+  (if (and (equal folder "") (stringp vm-last-visit-pop-folder))
+      (setq folder vm-last-visit-pop-folder))
+  (if (null (vm-pop-find-spec-for-name folder))
+      (error "No such POP folder: %s" folder))
+  (setq vm-last-visit-pop-folder folder)
+  (vm folder read-only 'pop))
+
+;;;###autoload
+(defun vm-visit-pop-folder-other-frame (folder &optional read-only)
+  "Like vm-visit-pop-folder, but run in a newly created frame."
+  (interactive
+   (save-excursion
+     (vm-session-initialization)
+     (vm-check-for-killed-folder)
+     (vm-select-folder-buffer-if-possible)
+     (let ((completion-list (mapcar (function (lambda (x) (nth 1 x)))
+				    vm-pop-folder-alist))
+	   (default vm-last-visit-pop-folder)
+	   (this-command this-command)
+	   (last-command last-command))
+       (list (vm-read-string
+	      (format "Visit%s POP folder:%s "
+		      (if current-prefix-arg " read only" "")
+		      (if default
+			  (format " (default %s)" default)
+			""))
+	      completion-list)
+	     current-prefix-arg))))
+  (vm-session-initialization)
+  (if (vm-multiple-frames-possible-p)
+      (vm-goto-new-frame 'folder))
+  (let ((vm-frame-per-folder nil)
+	(vm-search-other-frames nil))
+    (vm-visit-pop-folder folder read-only))
+  (if (vm-multiple-frames-possible-p)
+      (vm-set-hooks-for-frame-deletion)))
+
+;;;###autoload
+(defun vm-visit-pop-folder-other-window (folder &optional read-only)
+  "Like vm-visit-pop-folder, but run in a different window."
+  (interactive
+   (save-excursion
+     (vm-session-initialization)
+     (vm-check-for-killed-folder)
+     (vm-select-folder-buffer-if-possible)
+     (let ((completion-list (mapcar (function (lambda (x) (nth 1 x)))
+				    vm-pop-folder-alist))
+	   (default vm-last-visit-pop-folder)
+	   (this-command this-command)
+	   (last-command last-command))
+       (list (vm-read-string
+	      (format "Visit%s POP folder:%s "
+		      (if current-prefix-arg " read only" "")
+		      (if default
+			  (format " (default %s)" default)
+			""))
+	      completion-list)
+	     current-prefix-arg))))
+  (vm-session-initialization)
+  (if (one-window-p t)
+      (split-window))
+  (other-window 1)
+  (let ((vm-frame-per-folder nil)
+	(vm-search-other-frames nil))
+    (vm-visit-pop-folder folder read-only)))
+
 (put 'vm-virtual-mode 'mode-class 'special)
 
 (defun vm-virtual-mode (&rest ignored)
@@ -882,8 +1016,7 @@ vm-visit-virtual-folder.")
     (if vm-raise-frame-at-startup
 	(vm-raise-frame))
     (vm-display nil nil (list this-command) (list this-command 'startup))
-    (and (vm-toolbar-support-possible-p) vm-use-toolbar
-	 (vm-toolbar-install-toolbar))
+    (vm-toolbar-install-or-uninstall-toolbar)
     (if first-time
 	(progn
 	  (if (vm-should-generate-summary)
@@ -1066,8 +1199,7 @@ summary buffer to select a folder."
 	;; need to do this after any frame creation because the
 	;; toolbar sets frame-specific height and width specifiers.
 	(set-buffer vm-folders-summary-buffer)
-	(and (vm-toolbar-support-possible-p) vm-use-toolbar
-	     (vm-toolbar-install-toolbar)))
+	(vm-toolbar-install-or-uninstall-toolbar))
     (vm-display nil nil '(vm-folders-summarize)
 		(list this-command)))
   (vm-update-summary-and-mode-line))
@@ -1119,8 +1251,7 @@ summary buffer to select a folder."
   "Submit a bug report, with pertinent information to the VM bug list."
   (interactive)
   (require 'reporter)
-  (require 'vm-version)
-  (require 'vm-vars)
+  (vm-session-initialization)
   ;; Use VM to send the bug report.  Could be trouble if vm-mail
   ;; is what the user wants to complain about.  But most of the
   ;; time we'll be fine and users like to use MIME to attach
@@ -1188,8 +1319,12 @@ summary buffer to select a folder."
       'vm-frame-per-summary
       'vm-highlight-url-face
       'vm-highlighted-header-regexp
+      'vm-honor-mime-content-disposition
       'vm-honor-page-delimiters
+      'vm-icontopbm-program
       'vm-image-directory
+      'vm-imagemagick-convert-program
+      'vm-imagemagick-identify-program
 ;; IMAP passwords might be listed here
 ;;      'vm-imap-auto-expunge-alist
       'vm-imap-bytes-per-session
@@ -1215,11 +1350,11 @@ summary buffer to select a folder."
       'vm-mail-header-insert-date
       'vm-mail-header-insert-message-id
       'vm-mail-hook
-      'vm-make-crash-box-name
-      'vm-make-spool-file-name
       'vm-mail-check-interval
       'vm-mail-mode-hook
       'vm-mail-send-hook
+      'vm-make-crash-box-name
+      'vm-make-spool-file-name
       'vm-mime-7bit-composition-charset
       'vm-mime-8bit-composition-charset
       'vm-mime-8bit-text-transfer-encoding
@@ -1233,6 +1368,7 @@ summary buffer to select a folder."
       'vm-mime-base64-encoder-switches
       'vm-mime-button-face
       'vm-mime-button-format-alist
+      'vm-mime-charset-converter-alist
       'vm-mime-charset-font-alist
       'vm-mime-confirm-delete
       'vm-mime-decode-for-preview
@@ -1252,7 +1388,9 @@ summary buffer to select a folder."
       'vm-mime-qp-decoder-switches
       'vm-mime-qp-encoder-program
       'vm-mime-qp-encoder-switches
+      'vm-mime-require-mime-version-header
       'vm-mime-type-converter-alist
+      'vm-mime-use-image-strips
       'vm-mime-uuencode-decoder-program
       'vm-mime-uuencode-decoder-switches
       'vm-mode-hook
@@ -1274,6 +1412,8 @@ summary buffer to select a folder."
 ;;      'vm-pop-auto-expunge-alist
       'vm-pop-bytes-per-session
       'vm-pop-expunge-after-retrieving
+;; POP passwords might be listed here
+;;      'vm-pop-folder-alist
       'vm-pop-max-message-size
       'vm-pop-messages-per-session
       'vm-pop-md5-program
@@ -1313,8 +1453,14 @@ summary buffer to select a folder."
 ;;      'vm-spool-files
       'vm-spool-file-suffixes
       'vm-spooled-mail-waiting-hook
+      'vm-ssh-program
+      'vm-ssh-program-switches
+      'vm-ssh-remote-command
       'vm-startup-with-summary
       'vm-strip-reply-headers
+      'vm-stunnel-program
+      'vm-stunnel-program-switches
+      'vm-stunnel-random-data-method
       'vm-subject-significant-chars
       'vm-summary-format
       'vm-summary-highlight-face
@@ -1330,6 +1476,7 @@ summary buffer to select a folder."
       'vm-thread-using-subject
       'vm-toolbar-pixmap-directory
       'vm-trust-From_-with-Content-Length
+      'vm-uncompface-program
       'vm-undisplay-buffer-hook
       'vm-unforwarded-header-regexp
       'vm-url-browser
@@ -1399,6 +1546,7 @@ summary buffer to select a folder."
   (require 'vm)
   (vm-note-emacs-version)
   (vm-check-emacs-version)
+  (add-hook 'kill-emacs-hook 'vm-garbage-collect-global)
 ;;  (vm-set-debug-flags)
   ;; If this is the first time VM has been run in this Emacs session,
   ;; do some necessary preparations.
@@ -1412,6 +1560,25 @@ summary buffer to select a folder."
 	  (or (vm-load-window-configurations vm-window-configuration-file)
 	      (setq vm-window-configurations vm-default-window-configuration)))
 	(setq vm-buffers-needing-display-update (make-vector 29 0))
+	(if (if (fboundp 'find-face)
+		(find-face 'vm-monochrome-image)
+	      (facep 'vm-monochrome-image))
+	    nil
+	  (make-face 'vm-monochrome-image)
+	  (set-face-background 'vm-monochrome-image "white")
+	  (set-face-foreground 'vm-monochrome-image "black"))
+	(if (or (not vm-fsfemacs-p)
+		;; don't need this face under Emacs 21.
+		(fboundp 'image-type-available-p)
+		(facep 'vm-image-placeholder))
+	    nil
+	  (make-face 'vm-image-placeholder)
+	  (if (fboundp 'set-face-stipple)
+	      (set-face-stipple 'vm-image-placeholder
+				(list 16 16
+				      (concat "UU\377\377UU\377\377UU\377\377"
+					      "UU\377\377UU\377\377UU\377\377"
+					      "UU\377\377UU\377\377")))))
 	;; default value of vm-mime-button-face is 'gui-button-face
 	;; this face doesn't exist by default in FSF Emacs 19.34.
 	;; Create it and initialize it to something reasonable.
@@ -1427,6 +1594,16 @@ summary buffer to select a folder."
 		     ;; names may not be valid.
 		     (set-face-foreground 'gui-button-face "white")
 		     (set-face-background 'gui-button-face "red")))))
+	;; gui-button-face might not exist under XEmacs either.
+	;; This can happen if XEmacs is built without window
+	;; system support.  In any case, create it anyway.
+	(if (and vm-xemacs-p (not (find-face 'gui-button-face)))
+	    (progn
+	      (make-face 'gui-button-face)
+	      (set-face-foreground 'gui-button-face "black" nil '(win))
+	      (set-face-background 'gui-button-face "gray75" nil '(win))
+	      (set-face-foreground 'gui-button-face "white" nil '(tty))
+	      (set-face-background 'gui-button-face "red" nil '(tty))))
 	(and (vm-mouse-support-possible-p)
 	     (vm-mouse-install-mouse))
 	(and (vm-menu-support-possible-p)
