@@ -2,7 +2,7 @@
 
 (provide 'vm-version)
 
-(defconst vm-version "6.95"
+(defconst vm-version "7.04"
   "Version number of VM.")
 
 (defun vm-version ()
@@ -77,7 +77,7 @@
   (cond (vm-xemacs-p
 	 (memq 'win (device-matching-specifier-tag-list)))
 	(vm-fsfemacs-p
-	 (memq window-system '(x w32 win32)))))
+	 (memq window-system '(x mac w32 win32)))))
 
 (defun vm-menu-support-possible-p ()
   (cond (vm-xemacs-p
@@ -87,19 +87,22 @@
  
 (defun vm-toolbar-support-possible-p ()
   (or (and vm-xemacs-p (featurep 'toolbar))
-      (and vm-fsfemacs-p (fboundp 'tool-bar-mode))))
+      (and vm-fsfemacs-p (fboundp 'tool-bar-mode) (boundp 'tool-bar-map))))
 
 (defun vm-multiple-fonts-possible-p ()
   (cond (vm-xemacs-p
 	 (memq (device-type) '(x mswindows)))
 	(vm-fsfemacs-p
-	 (memq window-system '(x w32 win32)))))
+	 (memq window-system '(x mac w32 win32)))))
 
 (defun vm-images-possible-here-p ()
   (or (and vm-xemacs-p (memq (device-type) '(x mswindows)))
-      (and vm-fsfemacs-p window-system (fboundp 'image-type-available-p))))
+      (and vm-fsfemacs-p window-system
+	   (or (fboundp 'image-type-available-p)
+	       (and (stringp vm-imagemagick-convert-program)
+		    (stringp vm-imagemagick-identify-program))))))
 
 (defun vm-image-type-available-p (type)
   (if (fboundp 'image-type-available-p)
       (image-type-available-p type)
-    (featurep type)))
+    (or (featurep type) (eq type 'xbm))))
